@@ -27,7 +27,6 @@ module Mixlib
     READ_WAIT_TIME = 0.01
     READ_SIZE = 4096
     DEFAULT_READ_TIMEOUT = 600
-    DEFAULT_ENVIRONMENT = {'LC_ALL' => 'C'}
 
     if RUBY_PLATFORM =~ /mswin|mingw32|windows/
       require 'mixlib/shellout/windows'
@@ -41,22 +40,30 @@ module Mixlib
     attr_accessor :user
     attr_accessor :domain
     attr_accessor :password
+    # TODO remove
     attr_accessor :with_logon
 
+    # Whether to simulate logon as the user. Normally set via options passed to new
+    # Always enabled on windows
+    attr_accessor :login
+
     # Group the command will run as. Normally set via options passed to new
     attr_accessor :group
 
     # Working directory for the subprocess. Normally set via options to new
     attr_accessor :cwd
 
-    # An Array of acceptable exit codes. #error! uses this list to determine if
-    # the command was successful. Normally set via options to new
+    # An Array of acceptable exit codes. #error? (and #error!) use this list
+    # to determine if the command was successful. Normally set via options to new
     attr_accessor :valid_exit_codes
 
-    # When live_stream is set, stdout of the subprocess will be copied to it as
-    # the subprocess is running. For example, if live_stream is set to STDOUT,
-    # the command's output will be echoed to STDOUT.
-    attr_accessor :live_stream
+    # When live_stdout is set, the stdout of the subprocess will be copied to it
+    # as the subprocess is running.
+    attr_accessor :live_stdout
+
+    # When live_stderr is set, the stderr of the subprocess will be copied to it
+    # as the subprocess is running.
+    attr_accessor :live_stderr
 
     # ShellOut will push data from :input down the stdin of the subprocss.
     # Normally set via options passed to new.
@@ -81,7 +88,7 @@ module Mixlib
 
     # Environment variables that will be set for the subcommand. Refer to the
     # documentation of new to understand how ShellOut interprets this.
-    attr_reader :environment
+    attr_accessor :environment
 
     # The maximum time this command is allowed to run. Usually set via options
     # to new
@@ -122,16 +129,25 @@ module Mixlib
     #   subprocess. This only has an effect if you call +error!+ after
     #   +run_command+.
     # * +environment+: a Hash of environment variables to set before the command
-    #   is run. By default, the environment will *always* be set to 'LC_ALL' => 'C'
-    #   to prevent issues with multibyte characters in Ruby 1.8. To avoid this,
-    #   use :environment => nil for *no* extra environment settings, or
-    #   :environment => {'LC_ALL'=>nil, ...} to set other environment settings
-    #   without changing the locale.
+    #   is run.
     # * +timeout+: a Numeric value for the number of seconds to wait on the
     #   child process before raising an Exception. This is calculated as the
     #   total amount of time that ShellOut waited on the child process without
     #   receiving any output (i.e., IO.select returned nil). Default is 60
     #   seconds. Note: the stdlib Timeout library is not used.
+    # * +input+: A String of data to be passed to the subcommand. This is
+    #   written to the child process' stdin stream before the process is
+    #   launched. The child's stdin stream will be a pipe, so the size of input
+    #   data should not exceed the system's default pipe capacity (4096 bytes
+    #   is a safe value, though on newer Linux systems the capacity is 64k by
+    #   default).
+    # * +live_stream+: An IO or Logger-like object (must respond to the append
+    #   operator +<<+) that will receive data as ShellOut reads it from the
+    #   child process. Generally this is used to copy data from the child to
+    #   the parent's stdout so that users may observe the progress of
+    #   long-running commands.
+    # * +login+: Whether to simulate a login (set secondary groups, primary group, environment
+    #   variables etc) as done by the OS in an actual login
     # === Examples:
     # Invoke find(1) to search for .rb files:
     #   find = Mixlib::ShellOut.new("find . -name '*.rb'")
@@ -146,15 +162,16 @@ module Mixlib
     #   cmd = Mixlib::ShellOut.new("apachectl", "start", :user => 'www', :env => nil, :cwd => '/tmp')
     #   cmd.run_command # etc.
     def initialize(*command_args)
-      @stdout, @stderr = '', ''
-      @live_stream = nil
+      @stdout, @stderr, @process_status = '', '', ''
+      @live_stdout = @live_stderr = nil
       @input = nil
       @log_level = :debug
       @log_tag = nil
-      @environment = DEFAULT_ENVIRONMENT
+      @environment = {}
       @cwd = nil
       @valid_exit_codes = [0]
       @terminate_reason = nil
+      @timeout = nil
 
       if command_args.last.is_a?(Hash)
         parse_options(command_args.pop)
@@ -163,6 +180,18 @@ module Mixlib
       @command = command_args.size == 1 ? command_args.first : command_args
     end
 
+    # Returns the stream that both is being used by both live_stdout and live_stderr, or nil
+    def live_stream
+      live_stdout == live_stderr ? live_stdout : nil
+    end
+
+    # A shortcut for setting both live_stdout and live_stderr, so that both the
+    # stdout and stderr from the subprocess will be copied to the same stream as
+    # the subprocess is running.
+    def live_stream=(stream)
+      @live_stdout = @live_stderr = stream
+    end
+
     # Set the umask that the subprocess will have. If given as a string, it
     # will be converted to an integer by String#oct.
     def umask=(new_umask)
@@ -171,6 +200,7 @@ module Mixlib
 
     # The uid that the subprocess will switch to. If the user attribute was
     # given as a username, it is converted to a uid by Etc.getpwnam
+    # TODO migrate to shellout/unix.rb
     def uid
       return nil unless user
       user.kind_of?(Integer) ? user : Etc.getpwnam(user.to_s).uid
@@ -178,9 +208,11 @@ module Mixlib
 
     # The gid that the subprocess will switch to. If the group attribute is
     # given as a group name, it is converted to a gid by Etc.getgrnam
+    # TODO migrate to shellout/unix.rb
     def gid
-      return nil unless group
-      group.kind_of?(Integer) ? group : Etc.getgrnam(group.to_s).gid
+      return group.kind_of?(Integer) ? group : Etc.getgrnam(group.to_s).gid if group
+      return Etc.getpwuid(uid).gid if using_login?
+      return nil
     end
 
     def timeout
@@ -227,17 +259,21 @@ module Mixlib
       super
     end
 
-    # Checks the +exitstatus+ against the set of +valid_exit_codes+. If
-    # +exitstatus+ is not in the list of +valid_exit_codes+, calls +invalid!+,
-    # which raises an Exception.
+    # Checks the +exitstatus+ against the set of +valid_exit_codes+.
+    # === Returns
+    # +true+ if +exitstatus+ is not in the list of +valid_exit_codes+, false
+    # otherwise.
+    def error?
+      !Array(valid_exit_codes).include?(exitstatus)
+    end
+
+    # If #error? is true, calls +invalid!+, which raises an Exception.
     # === Returns
     # nil::: always returns nil when it does not raise
     # === Raises
     # ::ShellCommandFailed::: via +invalid!+
     def error!
-      unless Array(valid_exit_codes).include?(exitstatus)
-        invalid!("Expected process to exit with #{valid_exit_codes.inspect}, but received '#{exitstatus}'")
-      end
+      invalid!("Expected process to exit with #{valid_exit_codes.inspect}, but received '#{exitstatus}'") if error?
     end
 
     # Raises a ShellCommandFailed exception, appending the
@@ -282,7 +318,11 @@ module Mixlib
         when 'returns'
           self.valid_exit_codes = Array(setting)
         when 'live_stream'
-          self.live_stream = setting
+          self.live_stdout = self.live_stderr = setting
+        when 'live_stdout'
+          self.live_stdout = setting
+        when 'live_stderr'
+          self.live_stderr = setting
         when 'input'
           self.input = setting
         when 'logger'
@@ -292,14 +332,9 @@ module Mixlib
         when 'log_tag'
           self.log_tag = setting
         when 'environment', 'env'
-          # Set the LC_ALL from the parent process if the user wanted
-          # to use the default.
-          if setting && setting.has_key?("LC_ALL") && setting['LC_ALL'].nil?
-            setting['LC_ALL'] = ENV['LC_ALL']
-          end
-          # passing :environment => nil means don't set any new ENV vars
-          @environment = setting.nil? ? {} : @environment.dup.merge!(setting)
-
+          self.environment = setting || {}
+        when 'login'
+          self.login = setting
         else
           raise InvalidCommandOption, "option '#{option.inspect}' is not a valid option for #{self.class.name}"
         end
@@ -309,6 +344,9 @@ module Mixlib
     end
 
     def validate_options(opts)
+      if login && !user
+        raise InvalidCommandOption, "cannot set login without specifying a user"
+      end
       super
     end
   end
