@@ -111,6 +111,12 @@ module Mixlib
               when WAIT_TIMEOUT
                 # Kill the process
                 if (Time.now - start_wait) > timeout
+                  begin
+                    Process.kill(:KILL, process.process_id)
+                  rescue Errno::EIO
+                    logger.warn("Failed to kill timed out process #{process.process_id}") if logger
+                  end
+
                   raise Mixlib::ShellOut::CommandTimeout, "command timed out:\n#{format_for_exception}"
                 end
 
@@ -122,8 +128,8 @@ module Mixlib
             end
 
           ensure
-            CloseHandle(process.thread_handle)
-            CloseHandle(process.process_handle)
+            CloseHandle(process.thread_handle) if process.thread_handle
+            CloseHandle(process.process_handle) if process.process_handle
           end
 
         ensure
@@ -156,7 +162,7 @@ module Mixlib
           begin
             next_chunk = stdout_read.readpartial(READ_SIZE)
             @stdout << next_chunk
-            @live_stream << next_chunk if @live_stream
+            @live_stdout << next_chunk if @live_stdout
           rescue EOFError
             stdout_read.close
             open_streams.delete(stdout_read)
@@ -165,7 +171,9 @@ module Mixlib
 
         if ready.first.include?(stderr_read)
           begin
-            @stderr << stderr_read.readpartial(READ_SIZE)
+            next_chunk = stderr_read.readpartial(READ_SIZE)
+            @stderr << next_chunk
+            @live_stderr << next_chunk if @live_stderr
           rescue EOFError
             stderr_read.close
             open_streams.delete(stderr_read)
@@ -198,7 +206,6 @@ module Mixlib
         end
       end
 
-
       # cmd does not parse multiple quotes well unless the whole thing is wrapped up in quotes.
       # https://github.com/opscode/mixlib-shellout/pull/2#issuecomment-4837859
       # http://ss64.com/nt/syntax-esc.html
